open Test_data

let id = fun x -> x

let suffixize expr = 
  let rec suffixize_aux expr cont =
    match expr with
    | Add  (x, y) -> 
      suffixize_aux x (fun x -> suffixize_aux y (fun y -> cont (Add (x, y))))
    | Neg   x     -> suffixize_aux x (fun x -> cont (Neg x))
    | Const x     -> cont (Const x)
    | Var   x     -> cont (Var (x ^ "_suffix"))
  in
  suffixize_aux expr id

let subs expr = 
  let rec subs_aux expr cont =
    match expr with
    | Add  (x, y) -> 
      subs_aux x (fun x -> subs_aux y (fun y -> cont (Add (x, y))))
    | Neg   x     -> subs_aux x (fun x -> cont (Neg x))
    | Const x     -> cont (Const x)
    | Var   x     -> cont (if (dionysus = Var x) then apollo else Var x)
  in
  subs_aux expr id

let append_tr a b =
  let rec loop acc = function
    | [] -> acc
    | x::xs -> loop (x::acc) xs
  in
  loop b (List.rev a)

let collect expr =
  let rec collect_aux expr cont =
    match expr with
    | Add  (x, y) -> 
      collect_aux x (fun x -> collect_aux y (fun y -> cont (append_tr x y)))
    | Neg   x     -> collect_aux x cont
    | Const x     -> cont []
    | Var   x     -> cont [x]
  in
  collect_aux expr id

let rec increase c = company_increase id c
and company_increase cont (C deps) =
  let rec incr_deps cont = function
    | [] -> cont []
    | h::t -> incr_deps (fun ls -> cont ((dept_increase id h) :: ls)) t
  in
  C (incr_deps id deps)
and dept_increase cont (D (name, man, sunits)) =
  let man = employee_increase man in
  let rec incr_sunits cont = function
    | [] -> cont []
    | h::t -> incr_sunits (fun ls -> cont ((subunit_increase h) :: ls)) t
  in
  D (name, man, incr_sunits id sunits)
and subunit_increase = function
  | PU emp -> PU (employee_increase emp)
  | DU dept -> DU (dept_increase id dept)
and employee_increase (E (person, salary)) = 
  E (person, salary_increase salary)
and salary_increase (S salary) = S (salary *. 1.5)
