open Test_data
open Generic

module Performance :
sig
  val measure : ('a -> 'b) -> 'a -> float * 'b
end =
struct
  open Unix
  let measure func data =
    let t1 = times () in
    let res = func data in
    let t2 = times () in
    (t2.tms_utime -. t1.tms_utime), res
end

let suffixize s = s ^ "_suffix"
let incr (S r) = S (r *. 1.5)
let subs = function
  | Var x when Var x = dionysus -> apollo
  | x -> x

let id = fun x -> x

let subs = Transform.everywhere expr (Lift.lift expr subs) id
let suffixize = Transform.everywhere expr (Lift.lift var suffixize) id

let append_tr a b =
  let rec loop acc = function
    | [] -> acc
    | x::xs -> loop (x::acc) xs
  in
  loop b (List.rev a)

let collect =
  Query.everything
    expr
    (Lift_q.lift (List.fold_left append_tr []) var (fun v _ -> [v]))
    id

let increase = Transform.everywhere company (Lift.lift salary incr) id

let size = int_of_float (2.0 ** 20.0)

let test_expr = generate_expr 0 size
let test_company = generate_company 0 size

(*
let _ = print_string "Test expression: "
let _ = print_string (expr_to_string test_expr)
let _ = print_string "\n"

let _ = print_string "Test company:    "
let _ = print_string (company_to_string test_company)
let _ = print_string "\n"
*)

let print_expr_result res =
  print_string "Result: ";
  print_string (expr_to_string res);
  print_string "\n"

let ht, res = Performance.measure Handcoded.suffixize test_expr
let gt, res = Performance.measure suffixize test_expr
let _ = print_string "Suffixize:  "
let _ = print_float (gt /. ht)
let _ = print_string "\n"
(*let _ = print_expr_result res*)

let ht, res = Performance.measure Handcoded.subs test_expr
let gt, res = Performance.measure subs test_expr
let _ = print_string "Substitute: "
let _ = print_float (gt /. ht)
let _ = print_string "\n"
(*let _ = print_expr_result res*)

let ht, res = Performance.measure Handcoded.increase test_company
let gt, res = Performance.measure increase test_company
let _ = print_string "Increase:   "
let _ = print_float (gt /. ht)
let _ = print_string "\n"
(*let _ = print_string "Resilt: "
let _ = print_string (company_to_string res)
let _ = print_string "\n"*)


let ht, res = Performance.measure Handcoded.collect test_expr
let gt, res = Performance.measure collect test_expr
let _ = print_string "Collect:    "
let _ = print_float (gt /. ht)
let _ = print_string "\n"
(*let _ = print_string "Resilt: "
let _ = print_string (Aux.map_to_string (fun x -> x) res)
let _ = print_string "\n"*)
